import 'reflect-metadata';
export var META_KEY = 'TransKey';
export var getJsonProperty = function (target, propertyKey) {
    return typeof Reflect.getMetadata === 'function' ? Reflect.getMetadata(META_KEY, target, propertyKey) : undefined;
};
export var setProperty = function (value) {
    return typeof Reflect.metadata === 'function' && Reflect.metadata(META_KEY, value);
};
export var isObject = function (target) { return Object.prototype.toString.call(target) === '[object Object]'; };
export var isArray = function (target) { return Object.prototype.toString.call(target) === '[object Array]'; };
export var hasAnyNullOrUndefined = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return args.some(function (arg) { return arg === null || arg === undefined; });
};
export var isNotBasicType = function (target) { return ['string', 'number', 'boolean'].every(function (type) { return type !== typeof target; }); };
export var isInvalidDate = function (date) { return date instanceof Date && isNaN(date.getTime()); };
export function formatDate(timestamp, format) {
    if (format === void 0) { format = 'Y-M-D h:m:s'; }
    var date = new Date(timestamp);
    if (isInvalidDate(date)) {
        return timestamp;
    }
    var dateInfo = {
        Y: "" + date.getFullYear(),
        M: "" + (date.getMonth() + 1),
        D: "" + date.getDate(),
        h: date.getHours(),
        m: date.getMinutes(),
        s: date.getSeconds()
    };
    var formatNumber = function (n) { return (n >= 10 ? "" + n : "0" + n); };
    return format
        .replace('Y', dateInfo.Y)
        .replace('M', dateInfo.M)
        .replace('D', dateInfo.D)
        .replace('h', formatNumber(dateInfo.h))
        .replace('m', formatNumber(dateInfo.m))
        .replace('s', formatNumber(dateInfo.s));
}
export function getRandomInt(min, max) {
    if (!min || !max) {
        return 0;
    }
    var range = max - min;
    var rand = Math.random();
    return min + Math.round(rand * range);
}
export function getRandomString(length, chars) {
    if (chars === void 0) { chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890'; }
    if (!length) {
        return '';
    }
    return new Array(length).fill(0).reduce(function (res) { return "" + res + chars.charAt(Math.floor(Math.random() * chars.length)); }, '');
}
export function getRandomFloat(length) {
    if (!length) {
        return 0;
    }
    var decimal = getRandomInt(1, length - 2);
    var numStr = getRandomString(length, '123456789');
    var floatStr = parseFloat(numStr.split('').reduce(function (res, char, index) { return "" + res + (length - index === decimal ? '.' : '') + char; }, ''));
    if (floatStr !== floatStr) {
        return 0;
    }
    return floatStr;
}
